xquery version "3.1";

module namespace json="https://mollenhauer-edition.de/ns/conversions/json";

import module namespace tei2plain = "https://mollenhauer-edition.de/ns/conversions/tei2plain" at "tei2plain.xqm";
import module namespace selection = "https://mollenhauer-edition.de/ns/conversions/selection" at "selection.xqm";

import module namespace functx="http://www.functx.com";

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace tei="http://www.tei-c.org/ns/1.0";

declare option output:method "json";
declare option output:indent "yes";

(:~
 : This function produces JSON-output of transformations from TEI to plain text.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $title Title of the text represented in the TEI file.
 : @param $year Publication-Year of the text represented in the TEI file.
 : @param $fassungen The Fassungen which should be converted.
 : @param $doc The TEI-file.
 :)
declare function json:plain-text-single($title as xs:string, $year as xs:string, $fassungen as xs:string*, $doc as node())
{
    map {
        "metadata" : map {
            "title" : $title,
            "year" : $year
        },
        "versions" : array {
            for $fassung in $fassungen
            return
                map {
                    "metadata" : map {
                        "title" : concat('Fassung ', $fassung)
                    },
                    "data" : if (selection:check_fassung($doc, $fassung)) then
                        fn:string-join(tei2plain:complete-corr($doc, $fassung), ' ')
                    else "Diese Fassung existiert nicht."
                }
        }
    }
};

(:~
 : This function produces JSON-output of transformations from TEI to plain text for a series of documents.
 :
 : @author Max-Ferdinand Zeterberg
 : @version 1.0
 :
 : @param $path Directory, where the TEI-files are stored.
 : @param $filenames Names of the TEI-files.
 : @param $fassungen The Fassungen which should be converted.
 :)
declare function json:plain-text($path as xs:string, $filenames as xs:string+, $fassungen as xs:string*)
{
    array {
    for $filename in $filenames
    return
        let $doc := fn:doc(concat($path, '/', $filename))
        let $title := fn:normalize-space($doc//tei:titleStmt/tei:title[@type = 'main']/text())
        let $year := functx:get-matches(fn:normalize-space($doc//tei:witness[1]/tei:listWit[1]/tei:witness[1]/tei:desc[1]/text()), '\d\d\d\d')
        return
            if (exists($fassungen)) then
                json:plain-text-single($title, $year, $fassungen, $doc)
            else
                let $wits := $doc//tei:sourceDesc/tei:listWit/tei:witness/@xml:id
                return
                    json:plain-text-single($title, $year, $wits, $doc)
    }
};