import React from 'react'
import Mark from 'mark.js'

import {locale, otherLocale, tease, t} from './kmg'

const Url = require('@wendig/lib').Url

class ArchHighlighter extends React.Component {
  constructor(props) {
    super(props)

    this.update = this.update.bind(this)
    this.toggle = this.toggle.bind(this)
    this.transLoaded = this.transLoaded.bind(this)

    this.active = true
    this.highlights = Url.current().hashParams()['highlight']

    if (this.highlights) {
      // this.highlights = decodeURIComponent(this.highlights).split('|')
      this.highlights = decodeURIComponent(this.highlights)
    }

    this.before = decodeURI(Url.current().hashParams()['before'])
    this.after = decodeURI(Url.current().hashParams()['after'])

    this.mark = new Mark(this.target())
  }

  componentDidMount() {
    // the synoptic text is not available on page load, the code loading it
    // emits the trans-loaded event on window when the text is avaible
    window.addEventListener('trans-loaded', this.transLoaded)
    this.buttonTarget().addEventListener('click', this.toggle)

    this.update()
  }

  componentWillUnmount() {
    this.buttonTarget().removeEventListener('click', this.toggle)
    window.removeEventListener('trans-loaded', this.transLoaded)
  }

  //activates the button and does the actual highlighting when active and when
  update() {

    this.buttonTarget().classList.toggle('d-none', !this.highlights)

    if (this.highlights && this.active) {

      // context before: shorten / mask brackets 
      var slice_before = this.before.slice(-20, )
      var slice_before = slice_before.replaceAll("(", "\\(")
      var slice_before = slice_before.replaceAll(")", "\\)")

      // context after: shorten / mask brackets 
      var slice_after = this.after.slice(0, 20)
      var slice_after = slice_after.replaceAll("(", "\\(")
      var slice_after = slice_after.replaceAll(")", "\\)")

      // build regex for complete string (positive lookbehind/lookahead for context)
      const regex_string = "(?<=" + slice_before + ")" + "(" + this.highlights + ")" + "(?=" + slice_after + ")"
      // insert regex to find all overlong whitespace passages
      const regex_string_whitespace_chars = regex_string.replaceAll(/\s+/gm, "\\s+")
      // set regex
      var re = new RegExp(regex_string_whitespace_chars, "gm")

      // handover regex to mark (use of markRegExp function)
      this.mark.markRegExp(re, {
        element: 'span',
        className: 'hl',
        acrossElements: true,
        "ignoreGroups": 0
        //debug: true
      })

      // find span with class "hl"
      // if span doesn't exist: repeat process only for context before highlight
      // (reason for non-findings: inserted models break the regex;
      // content of modals is excluded in ES index and cannot taken into account here)
      var span_hl = document.getElementsByClassName("hl");
      if (span_hl.length == 0) {
        const regex_string_before = "(?<=" + slice_before + ")" + "(" + this.highlights + ")"
        const regex_string_before_whitespace_chars = regex_string_before.replaceAll(/\s+/gm, "\\s+")
        var re = new RegExp(regex_string_before_whitespace_chars, "gm")
        this.mark.markRegExp(re, {
          element: 'span',
          className: 'hl',
          acrossElements: true,
          "ignoreGroups": 0
          //debug: true
        })
        var span_hl = document.getElementsByClassName("hl");
      }
      // find span with class "hl"
      // if span doesn't exist: repeat process only for context after highlight
      if (span_hl.length == 0) {
        const regex_string_after = "(" + this.highlights + ")" + "(?=" + slice_after + ")"
        const regex_string_after_whitespace_chars = regex_string_after.replaceAll(/\s+/gm, "\\s+")
        var re = new RegExp(regex_string_after_whitespace_chars, "gm")
        this.mark.markRegExp(re, {
          element: 'span',
          className: 'hl',
          acrossElements: true,
          "ignoreGroups": 0
          //debug: tru
        })
        var span_hl = document.getElementsByClassName("hl");
      }
      // find span with class "hl"
      // if span doesn't exist: only mark highlight 
      if (span_hl.length == 0) {
        const regex_string_only_highlights = this.highlights
        const regex_string_only_highlights_whitespace_chars = regex_string_only_highlights.replaceAll(/\s+/gm, "\\s+")
        var re = new RegExp(regex_string_only_highlights_whitespace_chars, "gm")
        this.mark.markRegExp(re, {
          element: 'span',
          className: 'hl',
          acrossElements: true,
          "ignoreGroups": 0
          //debug: tru
        })
      }

      // scroll to highlight
      span_hl[0].scrollIntoView(
        // false
        // {
        // behavior: "smooth",
        // block: "start",
        // inline: "nearest"
        // }
        {
          behavior: 'auto',
          block: 'center',
          inline: 'center'
        }
      );
    } else {
      this.mark.unmark()
    }
  }

  toggle(event) {
    event.preventDefault()
    this.active = !this.active

    this.buttonTarget().querySelector('a').classList.toggle('text-muted')
    this.update()
  }

  transLoaded(event) {
    this.mark = new Mark(this.target())
    this.update()
  }

  target() {
    return document.querySelector('#edition, #wiki')
  }

  buttonTarget() {
    return document.querySelector('.arch-highlight-toggle')
  }

  render() {
    return ''
  }
}

export default ArchHighlighter
